@@ -30,6 +30,7 @@ const Main = imports.ui.main;
 const BoxPointer = imports.ui.boxpointer;
 const PopupMenu = imports.ui.popupMenu;
 const GrabHelper = imports.ui.grabHelper;
+const Config = imports.misc.config;
 
 const ExtensionUtils = imports.misc.extensionUtils;
 const Me = ExtensionUtils.getCurrentExtension();
@@ -58,6 +59,7 @@ var UndoStatus = {
 var StoredCoordinates = {
     PRESERVE: 0,
     OVERWRITE:1,
+    ASSIGN:2,
 };
 
 class Placeholder extends St.Bin {
@@ -208,6 +210,9 @@ var DesktopGrid = class {
     }
 
     _parseClipboardText(text) {
+        if (text === null)
+            return [false, false, null];
+
         let lines = text.split('\n');
         let [mime, action, ...files] = lines;
 
@@ -299,9 +304,7 @@ var DesktopGrid = class {
 
     _onOpenTerminalClicked() {
         let desktopPath = DesktopIconsUtil.getDesktopDir().get_path();
-        let command = DesktopIconsUtil.getTerminalCommand(desktopPath);
-
-        Util.spawnCommandLine(command);
+        DesktopIconsUtil.launchTerminal(desktopPath);
     }
 
     _syncUndoRedo() {
@@ -348,12 +351,13 @@ var DesktopGrid = class {
                     Clipboard.get_text(CLIPBOARD_TYPE,
                         (clipBoard, text) => {
                             let [valid, is_cut, files] = this._parseClipboardText(text);
-                            this._pasteMenuItem.actor.visible = valid;
+                            this._pasteMenuItem.setSensitive(valid);
                         }
                     );
                 }
             }
         );
+        this._pasteMenuItem.setSensitive(false);
 
         return menu;
     }
@@ -367,7 +371,7 @@ var DesktopGrid = class {
         this.actor._desktopBackgroundManager.ignoreRelease();
     }
 
-    _addFileItemTo(fileItem, column, row, overwriteCoordinates) {
+    _addFileItemTo(fileItem, column, row, coordinatesAction) {
         let placeholder = this.layout.get_child_at(column, row);
         placeholder.child = fileItem.actor;
         this._fileItems.push(fileItem);
@@ -381,18 +385,18 @@ var DesktopGrid = class {
          * Also store the new possition if it has been moved by the user,
          * and not triggered by a screen change.
          */
-        if ((fileItem.savedCoordinates == null) || (overwriteCoordinates == StoredCoordinates.OVERWRITE)) {
+        if ((fileItem.savedCoordinates == null) || (coordinatesAction == StoredCoordinates.OVERWRITE)) {
             let [fileX, fileY] = placeholder.get_transformed_position();
             fileItem.savedCoordinates = [Math.round(fileX), Math.round(fileY)];
         }
     }
 
-    addFileItemCloseTo(fileItem, x, y, overwriteCoordinates) {
-        let [column, row] = this._getEmptyPlaceClosestTo(x, y);
-        this._addFileItemTo(fileItem, column, row, overwriteCoordinates);
+    addFileItemCloseTo(fileItem, x, y, coordinatesAction) {
+        let [column, row] = this._getEmptyPlaceClosestTo(x, y, coordinatesAction);
+        this._addFileItemTo(fileItem, column, row, coordinatesAction);
     }
 
-    _getEmptyPlaceClosestTo(x, y) {
+    _getEmptyPlaceClosestTo(x, y, coordinatesAction) {
         let maxColumns = this._getMaxColumns();
         let maxRows = this._getMaxRows();
 
@@ -417,6 +421,8 @@ var DesktopGrid = class {
                     continue;
 
                 let [proposedX, proposedY] = placeholder.get_transformed_position();
+                if (coordinatesAction == StoredCoordinates.ASSIGN)
+                    return [column, row];
                 let distance = DesktopIconsUtil.distanceBetweenPoints(proposedX, proposedY, x, y);
                 if (distance < minDistance) {
                     found = true;
@@ -629,9 +635,12 @@ var RenamePopup = class {
         }
 
         this._boxPointer.setPosition(this._source.actor, 0.5);
-        this._boxPointer.show(BoxPointer.PopupAnimation.FADE |
-                              BoxPointer.PopupAnimation.SLIDE,
-                              null);
+        if (ExtensionUtils.versionCheck(['3.28', '3.30'], Config.PACKAGE_VERSION))
+            this._boxPointer.show(BoxPointer.PopupAnimation.FADE |
+                                  BoxPointer.PopupAnimation.SLIDE);
+        else
+            this._boxPointer.open(BoxPointer.PopupAnimation.FADE |
+                                  BoxPointer.PopupAnimation.SLIDE);
 
         this.emit('open-state-changed', true);
     }
@@ -642,8 +651,13 @@ var RenamePopup = class {
 
         this._grabHelper.ungrab({ actor: this.actor });
 
-        this._boxPointer.hide(BoxPointer.PopupAnimation.FADE |
-                              BoxPointer.PopupAnimation.SLIDE);
+        if (ExtensionUtils.versionCheck(['3.28', '3.30'], Config.PACKAGE_VERSION))
+            this._boxPointer.hide(BoxPointer.PopupAnimation.FADE |
+                                   BoxPointer.PopupAnimation.SLIDE);
+        else
+            this._boxPointer.close(BoxPointer.PopupAnimation.FADE |
+                                  BoxPointer.PopupAnimation.SLIDE);
+
         this._isOpen = false;
         this.emit('open-state-changed', false);
     }
