@@ -83,7 +83,7 @@ var DesktopManager = GObject.registerClass({
         this._monitorsChangedId = Main.layoutManager.connect('monitors-changed', () => this._recreateDesktopIcons());
         this._rubberBand = new St.Widget({ style_class: 'rubber-band' });
         this._rubberBand.hide();
-        Main.layoutManager.uiGroup.add_child(this._rubberBand);
+        Main.layoutManager._backgroundGroup.add_child(this._rubberBand);
         this._grabHelper = new GrabHelper.GrabHelper(global.stage);
 
         this._addDesktopIcons();
@@ -96,6 +96,7 @@ var DesktopManager = GObject.registerClass({
         });
 
         this._selection = new Set();
+        this._currentSelection = new Set();
         this._inDrag = false;
         this._dragXStart = Number.POSITIVE_INFINITY;
         this._dragYStart = Number.POSITIVE_INFINITY;
@@ -145,7 +146,7 @@ var DesktopManager = GObject.registerClass({
                 y0 = y;
             }
             for (let [fileUri, fileItem] of this._fileItems) {
-                fileItem.emit('selected', true,
+                fileItem.emit('selected', true, true,
                               fileItem.intersectsWith(x0, y0, x1 - x0, y1 - y0));
             }
         });
@@ -159,6 +160,9 @@ var DesktopManager = GObject.registerClass({
         global.stage.disconnect(this._stageReleaseEventId);
         this._rubberBandId = 0;
         this._stageReleaseEventId = 0;
+
+        this._selection = new Set([...this._selection, ...this._currentSelection]);
+        this._currentSelection.clear();
     }
 
     _updateRubberBand(currentX, currentY) {
@@ -585,11 +589,18 @@ var DesktopManager = GObject.registerClass({
     }
 
     _addFileItemCloseTo(item) {
-        let [x, y] = (item.savedCoordinates == null) ? [0, 0] : item.savedCoordinates;
+        let coordinates;
+        let x = 0;
+        let y = 0;
+        let coordinatesAction = DesktopGrid.StoredCoordinates.ASSIGN;
+        if (item.savedCoordinates != null) {
+            [x, y] = item.savedCoordinates;
+            coordinatesAction = DesktopGrid.StoredCoordinates.PRESERVE;
+        }
         let monitorIndex = findMonitorIndexForPos(x, y);
         let desktopGrid = this._desktopGrids[monitorIndex];
         try {
-            desktopGrid.addFileItemCloseTo(item, x, y, DesktopGrid.StoredCoordinates.PRESERVE);
+            desktopGrid.addFileItemCloseTo(item, x, y, coordinatesAction);
         } catch (e) {
             log(`Error adding children to desktop: ${e.message}`);
         }
@@ -656,24 +667,26 @@ var DesktopManager = GObject.registerClass({
         });
     }
 
-    _onFileItemSelected(fileItem, keepCurrentSelection, addToSelection) {
+    _onFileItemSelected(fileItem, keepCurrentSelection, rubberBandSelection, addToSelection) {
 
         if (!keepCurrentSelection && !this._inDrag)
             this.clearSelection();
 
+        let selection = keepCurrentSelection && rubberBandSelection ? this._currentSelection : this._selection;
         if (addToSelection)
-            this._selection.add(fileItem);
+            selection.add(fileItem);
         else
-            this._selection.delete(fileItem);
+            selection.delete(fileItem);
 
         for (let [fileUri, fileItem] of this._fileItems)
-            fileItem.isSelected = this._selection.has(fileItem);
+            fileItem.isSelected = this._currentSelection.has(fileItem) || this._selection.has(fileItem);
     }
 
     clearSelection() {
         for (let [fileUri, fileItem] of this._fileItems)
             fileItem.isSelected = false;
         this._selection = new Set();
+        this._currentSelection = new Set();
     }
 
     _getClipboardText(isCopy) {
