@@ -183,12 +183,16 @@ var FileItem = class {
             this._loadThumbnailDataCancellable.cancel();
 
         /* Desktop file */
-        if (this._monitorDesktopFileId)
+        if (this._monitorDesktopFileId) {
             this._monitorDesktopFile.disconnect(this._monitorDesktopFileId);
+            this._monitorDesktopFile.cancel();
+        }
 
         /* Trash */
-        if (this._monitorTrashDir)
+        if (this._monitorTrashDir) {
             this._monitorTrashDir.disconnect(this._monitorTrashId);
+            this._monitorTrashDir.cancel();
+        }
         if (this._queryTrashInfoCancellable)
             this._queryTrashInfoCancellable.cancel();
         if (this._scheduleTrashRefreshId)
@@ -539,10 +543,6 @@ var FileItem = class {
                         log(`Failed to set unix mode: ${e.message}`);
                     }
             });
-            this._file.set_attributes_from_info(info,
-                                                Gio.FileQueryInfoFlags.NONE,
-                                                GLib.PRIORITY_LOW,
-                                                null);
         }
     }
 
@@ -612,15 +612,14 @@ var FileItem = class {
     }
 
     _onOpenTerminalClicked () {
-        let command = DesktopIconsUtil.getTerminalCommand(this.file.get_path());
-        Util.spawnCommandLine(command);
+        DesktopIconsUtil.launchTerminal(this.file.get_path());
     }
 
     _onPressButton(actor, event) {
         let button = event.get_button();
         if (button == 3) {
             if (!this.isSelected)
-                this.emit('selected', false, true);
+                this.emit('selected', false, false, true);
             this._menu.toggle();
             if (this._actionOpenWith) {
                 let allowOpenWith = (Extension.desktopManager.getNumberOfSelectedItems() == 1);
@@ -643,7 +642,7 @@ var FileItem = class {
                 let shiftPressed = !!(event.get_state() & Clutter.ModifierType.SHIFT_MASK);
                 let controlPressed = !!(event.get_state() & Clutter.ModifierType.CONTROL_MASK);
                 if (!this.isSelected) {
-                    this.emit('selected', shiftPressed || controlPressed, true);
+                    this.emit('selected', shiftPressed || controlPressed, false, true);
                 }
             }
             return Clutter.EVENT_STOP;
@@ -686,7 +685,7 @@ var FileItem = class {
                 let controlPressed = !!(event.get_state() & Clutter.ModifierType.CONTROL_MASK);
                 if ((event.get_click_count() == 1) && Prefs.CLICK_POLICY_SINGLE && !shiftPressed && !controlPressed)
                     this.doOpen();
-                this.emit('selected', shiftPressed || controlPressed, true);
+                this.emit('selected', shiftPressed || controlPressed, false, true);
                 return Clutter.EVENT_STOP;
             }
             if ((event.get_click_count() == 2) && (!Prefs.CLICK_POLICY_SINGLE))
